
# compute schurz

def PRing_two_groups_of_variables(letter1, letter2, n1, n2):
   var_names_1 = [letter1 + ('%d' % j) for j in range(n1)]
   var_names_2 = [letter2 + ('%d' % j) for j in range(n2)]
   var_names = var_names_1 + var_names_2
   return PolynomialRing(QQ, var_names)

def myvars_two_groups(letter1, letter2, n1, n2):
   return PRing_two_groups_of_variables(letter1, letter2, n1, n2).gens()

def proddif(x, j):
   return prod(x[j] - x[k] for k in range(j) + range(j + 1, len(x)))

def hom_via_sum_of_quotients(m, x):
   n = len(x)
   return sum((x[j] ** (m + n - 1)) / proddif(x, j) for j in range(n))

def homz_via_sum_of_quotients_ChebU(m, z):
   n = len(z)
   return sum(chebyshev_U(m + n - 1, z[j] / 2) / proddif(z, j) for j in range(n))

def coef_sum_chebU(m, k):
   jmax = floor((m - k) / 2)
   return sum(((- 1) ** j) * binomial(k + j, j) for j in range(jmax + 1))

def sum_chebU(m, t):
   return sum(coef_sum_chebU(m, k) * (t ** k) for k in range(m + 1))

def homz_odd(m, z):
   n = len(z)
   return sum(sum_chebU(n + m - 1, z[j]) / proddif(z, j) for j in range(n))

def schur(la, y):
   n = len(la)
   JT = matrix(y[0].parent(), n, n)
   for j in range(n):
      for k in range(n):
         m = la[j] - j + k
         JT[j, k] = hom_via_sum_of_quotients(m, y) if m >= 0 else 0
   return JT.det()

def schurz(la, z):
   n = len(la)
   JT = matrix(z[0].parent(), n, n)
   for j in range(n):
      for k in range(n):
         m = la[j] - j + k
         JT[j, k] = homz_via_sum_of_quotients_ChebU(m, z) if m >= 0 else 0
   return JT.det()

def schurz_odd(la, z):
   n = len(la)
   JT = matrix(z[0].parent(), n, n)
   for j in range(n):
      for k in range(n):
         m = la[j] - j + k
         JT[j, k] = homz_odd(m, z) if m >= 0 else 0
   return JT.det()

# tests

def test_cauchy_schurz_schur_dual_pol(n1, n2, verbos):
   zy = myvars_two_groups('z', 'y', n1, n2)
   z = zy[0 : n1]
   y = zy[n1 : n1 + n2]
   ps = []
   for la_weight in range(2 * n1 * n2 + 1):
      ps += Partitions(la_weight, max_length = 2 * n1, max_part = n2).list()
   if verbos >= 2:
      print('test_cauchy_schurz_schur_dual_pol, n1 = %d, n2 = %d' % (n1, n2))
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz(la, z)
      s2 = schur(la.conjugate(), y)
      lhs += s1 * s2
   rhs = prod(1 + z[j] * y[k] + y[k] * y[k] for j in range(n1) for k in range(n2))
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_schur_dual_pol(n1max, n2max, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_schur_dual_pol, n1max = %d, n2max = %d' % (n1max, n2max))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_schur_dual_pol(n1, n2, verbos) for (n1, n2) in tasks)

def test_cauchy_schurz_schurz_dual_pol(n1, n2, verbos):
   zw = myvars_two_groups('z', 'w', n1, n2)
   z = zw[0 : n1]
   w = zw[n1 : n1 + n2]
   ps = []
   for la_weight in range(4 * n1 * n2 + 1):
      ps += Partitions(la_weight, max_length = 2 * n1, max_part = 2 * n2).list()
   if verbos >= 2:
      print('test_cauchy_schurz_schurz_dual_pol, n1 = %d, n2 = %d' % (n1, n2))
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz(la, z)
      s2 = schurz(la.conjugate(), w)
      lhs += s1 * s2
   rhs = prod((z[j] + w[k]) ** 2 for j in range(n1) for k in range(n2))
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_schurz_dual_pol(n1max, n2max, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_schurz_dual, n1max = %d, n2max = %d' % (n1max, n2max))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_schurz_dual_pol(n1, n2, verbos) for (n1, n2) in tasks)

def test_cauchy_schurz_odd_schur_dual_pol(n1, n2, verbos):
   zy = myvars_two_groups('z', 'y', n1, n2)
   z = zy[0 : n1]
   y = zy[n1 : n1 + n2]
   ps = []
   for la_weight in range((2 * n1 + 1) * n2 + 1):
      ps += Partitions(la_weight, max_length = 2 * n1 + 1, max_part = n2).list()
   if verbos >= 2:
      print('test_cauchy_schurz_odd_schur_dual_pol, n1 = %d, n2 = %d' % (n1, n2))
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz_odd(la, z)
      s2 = schur(la.conjugate(), y)
      lhs += s1 * s2
   prod1 = prod(1 + z[j] * y[k] + y[k] * y[k] for j in range(n1) for k in range(n2))
   prod2 = prod(1 + y[k] for k in range(n2))
   rhs = prod1 * prod2
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_odd_schur_dual_pol(n1max, n2max, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_odd_schur_dual_pol, n1max = %d, n2max = %d' % (n1max, n2max))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_odd_schur_dual_pol(n1, n2, verbos) for (n1, n2) in tasks)

def test_cauchy_schurz_odd_schurz_odd_dual_pol(n1, n2, verbos):
   zw = myvars_two_groups('z', 'w', n1, n2)
   z = zw[0 : n1]
   w = zw[n1 : n1 + n2]
   ps = []
   for la_weight in range((2 * n1 + 1) * (2 * n2 + 1) + 1):
      ps += Partitions(la_weight, max_length = 2 * n1 + 1, max_part = 2 * n2 + 1).list()
   if verbos >= 2:
      print('test_cauchy_schurz_odd_schurz_odd_dual_pol, n1 = %d, n2 = %d' % (n1, n2))
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz_odd(la, z)
      s2 = schurz_odd(la.conjugate(), w)
      lhs += s1 * s2
   prod1 = prod((z[j] + w[k]) ** 2 for j in range(n1) for k in range(n2))
   prod2 = prod(2 + z[j] for j in range(n1))
   prod3 = prod(2 + w[k] for k in range(n2))
   rhs = 2 * prod1 * prod2 * prod3
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_odd_schurz_odd_dual_pol(n1max, n2max, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_odd_schurz_odd_dual_pol, n1max = %d, n2max = %d' % (n1max, n2max))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_odd_schurz_odd_dual_pol(n1, n2, verbos) for (n1, n2) in tasks)

def polynomial_restrict_degrees(f, dmax):
   PR = f.parent()
   v = PR.gens()
   result = PR.zero()
   for c, mon in list(f):
      if mon.degree() <= dmax:
         result += c * mon
   return result

def geom_progr_first_terms(q, maxpower):
   return sum(q ** p for p in range(maxpower + 1))

def test_cauchy_schurz_schur_pol(n1, n2, maxdeg, verbos):
   zy = myvars_two_groups('z', 'y', n1, n2)
   z = zy[0 : n1]
   y = zy[n1 : n1 + n2]
   ps = []
   my_max_length = min(2 * n1, n2)
   for la_weight in range(maxdeg + 1):
      ps += Partitions(la_weight, max_length = my_max_length).list()
   if verbos >= 2:
      print('test_cauchy_schurz_schur_pol, n1 = %d, n2 = %d, maxdeg = %d' % (n1, n2, maxdeg))
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz(la, z)
      s2 = schur(la, y)
      lhs += s1 * s2
   lhs = polynomial_restrict_degrees(lhs, maxdeg)
   rhs = z[0].parent().one()
   for j in range(n1):
      for k in range(n2):
         rhs = rhs * geom_progr_first_terms(z[j] * y[k] - y[k] * y[k], maxdeg)
         rhs = polynomial_restrict_degrees(rhs, maxdeg)
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_schur_pol(n1max, n2max, maxdeg, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_schur_pol, n1max = %d, n2max = %d, maxdeg = %d' % (n1max, n2max, maxdeg))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_schur_pol(n1, n2, maxdeg, verbos) for (n1, n2) in tasks)

def test_cauchy_schurz_odd_schur_pol(n1, n2, maxdeg, verbos):
   if verbos >= 2:
      print('test_cauchy_schurz_odd_schur_pol, n1 = %d, n2 = %d, maxdeg = %d' % (n1, n2, maxdeg))
   zy = myvars_two_groups('z', 'y', n1, n2)
   z = zy[0 : n1]
   y = zy[n1 : n1 + n2]
   ps = []
   my_max_length = min(2 * n1 + 1, n2)
   for la_weight in range(maxdeg + 1):
      ps += Partitions(la_weight, max_length = my_max_length).list()
   if verbos >= 2:
      print('number of partitions = %d' % len(ps))
   lhs = z[0].parent().zero()
   for la in ps:
      s1 = schurz_odd(la, z)
      s2 = schur(la, y)
      lhs += s1 * s2
   lhs = polynomial_restrict_degrees(lhs, maxdeg)
   prod1 = z[0].parent().one()
   for j in range(n1):
      for k in range(n2):
         prod1 = prod1 * geom_progr_first_terms(z[j] * y[k] - y[k] * y[k], maxdeg)
         prod1 = polynomial_restrict_degrees(prod1, maxdeg)
   prod2 = z[0].parent().one()
   for k in range(n2):
      prod2 = prod2 * geom_progr_first_terms(y[k], maxdeg)
      prod2 = polynomial_restrict_degrees(prod2, maxdeg)
   rhs = polynomial_restrict_degrees(prod1 * prod2, maxdeg)
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_cauchy_schurz_odd_schur_pol(n1max, n2max, maxdeg, verbos):
   if verbos >= 1:
      print('big_test_cauchy_schurz_odd_schur_pol, n1max = %d, n2max = %d, maxdeg = %d' % (n1max, n2max, maxdeg))
   tasks = [(n1, n2) for n1 in range(1, n1max + 1) for n2 in range(1, n2max + 1)]
   return all(test_cauchy_schurz_odd_schur_pol(n1, n2, maxdeg, verbos) for (n1, n2) in tasks)

import time

def all_big_tests_cauchy_schurz_pol():
   verbos = 2
   t = time.time()
   res = [0] * 6
   res[0] = big_test_cauchy_schurz_schur_pol(3, 2, 15, verbos)
   res[1] = big_test_cauchy_schurz_schur_dual_pol(3, 4, verbos)
   res[2] = big_test_cauchy_schurz_schurz_dual_pol(3, 3, verbos)
   res[3] = big_test_cauchy_schurz_odd_schur_pol(3, 2, 15, verbos)
   res[4] = big_test_cauchy_schurz_odd_schur_dual_pol(3, 4, verbos)
   res[5] = big_test_cauchy_schurz_odd_schurz_odd_dual_pol(3, 2, verbos)
   total_result = all(res)
   t = time.time() - t
   print('time = %.2f seconds' % t)
   print('total result = ' + str(total_result))

# time < 300 sec
all_big_tests_cauchy_schurz_pol()

